import requests
import os
import flask
from vex.subsystem.aplicacao.tabela_preco.resource import TabelaPreco
from viggocore.common.subsystem import operation, manager


class CadastrarEmpresa(operation.Create):

    def pre(self, session, **kwargs):
        return True
    
    def do(self, session, **kwargs):
        url = self.manager.get_endpoint('/empresas')
        return self.manager.executar_requisicao('POST', url, json=kwargs)


class ListarEmpresas(operation.List):

    def do(self, session, **kwargs):
        url = self.manager.get_endpoint('/empresas')
        return self.manager.executar_requisicao('GET', url, params=kwargs)


class BuscarEmpresa(operation.List):

    def do(self, session, **kwargs):
        cpf_cnpj = self.manager.get_cpf_cnpj(**kwargs)
        url = self.manager.get_endpoint(f'/empresas/{cpf_cnpj}')
        return self.manager.executar_requisicao('GET', url, params=kwargs)


class AlterarEmpresa(operation.Update):

    def pre(self, session, **kwargs):
        return True

    def do(self, session, **kwargs):
        cpf_cnpj = self.manager.get_cpf_cnpj(**kwargs)
        url = self.manager.get_endpoint(f'/empresas/{cpf_cnpj}')
        return self.manager.executar_requisicao('PUT', url, json=kwargs)


class ConsultarCerticado(operation.List):

    def do(self, session, **kwargs):
        cpf_cnpj = self.manager.get_cpf_cnpj(**kwargs)
        url = self.manager.get_endpoint(f'/empresas/{cpf_cnpj}/certificado')
        return self.manager.executar_requisicao('GET', url)

class CadastrarCertificado(operation.Update):

    def pre(self, session, **kwargs):
        return True

    def do(self, session, **kwargs):
        cpf_cnpj = self.manager.get_cpf_cnpj(**kwargs)
        url = self.manager.get_endpoint(f'/empresas/{cpf_cnpj}/certificado')
        kwargs.pop('cpf_cnpj')
        return self.manager.executar_requisicao('PUT', url, json=kwargs)


class DeletarCertificado(operation.Delete):

    def pre(self, session, **kwargs):
        return True

    def do(self, session, **kwargs):
        cpf_cnpj = self.manager.get_cpf_cnpj(**kwargs)
        url = self.manager.get_endpoint(f'/empresas/{cpf_cnpj}/certificado')
        return self.manager.executar_requisicao('DELETE', url)


class ConsultarConfiguracaoNfe(operation.List):

    def pre(self, session, **kwargs):
        return True

    def do(self, session, **kwargs):
        cpf_cnpj = self.manager.get_cpf_cnpj(**kwargs)
        url = self.manager.get_endpoint(f'/empresas/{cpf_cnpj}/nfe')
        return self.manager.executar_requisicao('GET', url)


class AlterarConfiguracaoNfe(operation.Update):

    def pre(self, session, **kwargs):
        return True

    def do(self, session, **kwargs):
        cpf_cnpj = self.manager.get_cpf_cnpj(**kwargs)
        url = self.manager.get_endpoint(f'/empresas/{cpf_cnpj}/nfe')
        kwargs.pop('cpf_cnpj')
        return self.manager.executar_requisicao('PUT', url, json=kwargs)


class EmitirNfe(operation.Create):

    def pre(self, session, **kwargs):
        return True

    def do(self, session, **kwargs):
        url = self.manager.get_endpoint('/nfe')
        return self.manager.executar_requisicao('POST', url, json=kwargs)


class ListarNfe(operation.List):
    def do(self, session, **kwargs):
        url = self.manager.get_endpoint('/nfe')
        return self.manager.executar_requisicao('GET', url, params=kwargs)


class ConsultaStatusDoServicoNaSefazAutorizadora(operation.List):

    def do(self, session, **kwargs):
        url = self.manager.get_endpoint('/nfe/sefaz/status')
        return self.manager.executar_requisicao('GET', url, params=kwargs)


class ConsultarNfe(operation.List):

    def do(self, session, **kwargs):
        nfe_id = self.manager.get_nfe_id(**kwargs)
        url = self.manager.get_endpoint(f'/nfe/{nfe_id}')
        return self.manager.executar_requisicao('GET', url)


class BaixarXml(operation.List):

    def do(self, session, **kwargs):
        nfe_id = self.manager.get_nfe_id(**kwargs)
        url = self.manager.get_endpoint(f'/nfe/{nfe_id}/xml')
        return self.manager.executar_requisicao('GET', url)


class ConsultarCancelamentoNfe(operation.List):

    def do(self, session, **kwargs):
        nfe_id = self.manager.get_nfe_id(**kwargs)
        url = self.manager.get_endpoint(f'/nfe/{nfe_id}/cancelamento')
        return self.manager.executar_requisicao('GET', url)


class BaixarXmlCancelamentoNfe(operation.List):
    def do(self, session, **kwargs):
        nfe_id = self.manager.get_nfe_id(**kwargs)
        url = self.manager.get_endpoint(f'/nfe/{nfe_id}/cancelamento/xml')
        return self.manager.executar_requisicao('GET', url)


class CancelarNfeAutorizada(operation.List):

    def do(self, session, **kwargs):
        nfe_id = self.manager.get_nfe_id(**kwargs)
        url = self.manager.get_endpoint(f'/nfe/{nfe_id}/cancelamento')
        kwargs.pop('nfe_id')
        return self.manager.executar_requisicao('POST', url, json=kwargs)


class Manager(manager.Manager):
    # BASE_URL_PROD = 'https://api.nuvemfiscal.com.br'
    BASE_URL_HOMO = 'https://api.sandbox.nuvemfiscal.com.br'

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.cadastrar_empresa = CadastrarEmpresa(self)
        self.listar_empresas = ListarEmpresas(self)
        self.buscar_empresa = BuscarEmpresa(self)
        self.alterar_empresa = AlterarEmpresa(self)
        self.consultar_certicado = ConsultarCerticado(self)
        self.cadastrar_certificado = CadastrarCertificado(self)
        self.deletar_certificado = DeletarCertificado(self)
        self.consultar_configuracao_nfe = ConsultarConfiguracaoNfe(self)
        self.alterar_configuracao_nfe = AlterarConfiguracaoNfe(self)
        self.emitir_nfe = EmitirNfe(self)
        self.listar_nfe = ListarNfe(self)
        self.consulta_status_do_servico_na_sefaz_autorizadora = \
            ConsultaStatusDoServicoNaSefazAutorizadora(self)
        self.consultar_nfe = ConsultarNfe(self)
        self.baixar_xml = BaixarXml(self)
        self.consultar_cancelamento_nfe = ConsultarCancelamentoNfe(self)
        self.baixar_xml_cancelamento_nfe = BaixarXmlCancelamentoNfe(self)
        self.cancelar_nfe_autorizada = CancelarNfeAutorizada(self)

    def get_authorization(self):
        authorization = os.getenv('NUVEM_FISCAL_AUTHORIZATION', None)
        if authorization is None:
            raise exception.PreconditionFailed(
                'NUVEM_FISCAL_AUTHORIZATION é obrigatório!')
        return  {'Authorization': authorization}

    def get_cpf_cnpj(self, **kwargs):
        cpf_cnpj = kwargs.get('cpf_cnpj', None)
        if cpf_cnpj is None:
            raise exception.BadRequest('O campo cpf_cnpj é obrigatório.')
        return cpf_cnpj

    def get_nfe_id(self, **kwargs):
        nfe_id = kwargs.get('nfe_id', None)
        if nfe_id is None:
            raise exception.BadRequest('O campo nfe_id é obrigatório.')
        return nfe_id

    def get_endpoint(self, resource):
        return self.BASE_URL_HOMO + resource

    def executar_requisicao(self, method, endpoint, params={}, json={}, headers={}):
        headers.update(self.get_authorization())
        if method == 'GET':
            return requests.get(
                endpoint, params=params, json=json, headers=headers)
        elif method == 'POST':
            return requests.post(
                endpoint, params=params, json=json, headers=headers)
        elif method == 'PUT':
            return requests.put(
                endpoint, params=params, json=json, headers=headers)
        elif method == 'DELETE':
            return requests.delete(endpoint)
        else:
            raise exception.OperationBadRequest(
                'Método de requisição não permitido.')
    
    def montar_response_dict(self, response):
        try:
            response_dict = json.loads(response.text)
        except Exception:
            response_dict = {'error': response.text}
        return response_dict
